using System.Collections;
using System.Collections.Generic;
using System;
using System.Linq;
using UnityEngine;
using TMPro;

public class BlockHandler : MonoBehaviour
{
    public GameObject nBlockInput;
    public GameObject blockObject;
    public AnimateCircle animateCircle;
    public int maxNumBlocks { get; private set; }
    public float blockStackRuntime { get; private set; }

    TMP_Dropdown dropdown;
    List<GameObject> blockStack = new List<GameObject>();
    GameObject runningBlock;
    int currentBlockIndex = 0;

    private Vector3 blockHomePosition = new Vector3(-104f, -345f, 0f);
    // Start is called before the first frame update
    void Awake()
    {
        dropdown = this.GetComponent<TMP_Dropdown>();
        GameObject firstBlock = GetBlockObjectPrefab();
        blockStack.Add(firstBlock);
        blockStack[0].SetActive(true);
        runningBlock = blockStack[0];

        UpdateOptions();
        SetRunningBlockIndex();
        UpdateRunningBlock();
        UpdateBlockStackRuntime();
    }

    // Update is called once per frame
    void Update()
    {
        
    }

    public void UpdateOptions()
    {
        UpdateMaxNumBlocks();
        dropdown.ClearOptions();
        for (int i = 1; i < maxNumBlocks + 1; i++)
        {
            dropdown.AddOptions(new List<string> { i.ToString() });
        }
    }

    public void UpdateMaxNumBlocks()
    { 
        TMP_InputField inputField;
        inputField = nBlockInput.GetComponent<TMP_InputField>();
        maxNumBlocks = Int32.Parse(inputField.text);
        if (maxNumBlocks <= 0)
        {
            maxNumBlocks = 1;
        }        
    }

    public void SetRunningBlockIndex(int blockIndexInput = -1)
    {
        if (blockIndexInput == -1)
        {
            currentBlockIndex = dropdown.value;
        }
        else
        {
            currentBlockIndex = blockIndexInput;
        }
        Debug.Log("current block index: " + currentBlockIndex);
    }

    public void UpdateBlockStack()
    {
        bool isEmpty = !blockStack.Any();
        int numblockStack = 0;
        if (!isEmpty)
        {
            numblockStack = blockStack.Count;
        }
        if (maxNumBlocks > numblockStack)
        {
            for (int i = numblockStack; i < maxNumBlocks; i++)
            {
                blockStack.Add(GetBlockObjectPrefab());
                blockStack[i].SetActive(false);
            }
        }
        else if (maxNumBlocks < numblockStack)
        {
            for (int i = numblockStack; i > maxNumBlocks; i--)
            {
                blockStack.RemoveAt(i);
            }
        }
        SetRunningBlockIndex();
        SetRunningBlock();
        UpdateRunningBlock();
        UpdateBlockStackRuntime();
    }

    public void SetRunningBlock()
    {
        runningBlock.SetActive(false);
        runningBlock = blockStack[currentBlockIndex];
        runningBlock.SetActive(true);
    }

    public void UpdateRunningBlock()
    {
        animateCircle.blockObj = runningBlock;
    }

    public List<GameObject> GetBlockStack()
    {
        return blockStack;
    }

    public void UpdateBlockStackRuntime()
    {
        blockStackRuntime = 0;
        foreach (var blockObject in blockStack)
        {
            BlockObjectClass blockObjectClass = blockObject.GetComponent<BlockObjectClass>();
            blockStackRuntime += blockObjectClass.MaxRuntime;
        }
    }

    GameObject GetBlockObjectPrefab()
    {
        GameObject block = Instantiate(
            blockObject,
            new Vector3(0f, 0f, 0f),
            Quaternion.identity
            );
        block.transform.parent = this.transform;
        block.transform.localPosition = blockHomePosition;
        block.transform.localScale = new Vector3(1f, 1f, 0f);
        return block;
    }

}
